package com.broodsoft.grandeur;

import static com.broodsoft.grandeur.Handlers.noOpFailureHandler;
import static com.broodsoft.grandeur.Handlers.noOpListener;

import java.util.concurrent.Executor;
import java.util.concurrent.Executors;
import java.util.concurrent.ThreadFactory;

import javafx.beans.InvalidationListener;
import javafx.beans.value.ChangeListener;
import javafx.concurrent.Task;

public final class Tasks {
	public static final Executor DEFAULT_EXECUTOR = Executors.unconfigurableExecutorService(
		Executors.newSingleThreadExecutor(new ThreadFactory() {
			public Thread newThread(Runnable r) {
				Thread thread = new Thread(r);
				thread.setDaemon(true);
				return thread;
			}
		}
	));


	public static <T> void call(Task<T> task) {
		call(task, Handlers.<T>noOpSuccessHandler(), noOpFailureHandler(), DEFAULT_EXECUTOR);
	}

	public static <T> void call(Task<T> task, ChangeListener<T> successHandler) {
		call(task, successHandler, noOpFailureHandler(), DEFAULT_EXECUTOR);
	}

	public static <T> void call(Task<T> task, InvalidationListener successListener) {
		call(task, successListener, noOpListener(), DEFAULT_EXECUTOR);
	}

	public static <T> void call(Task<T> task, InvalidationListener successListener, InvalidationListener failureListener) {
		call(task, successListener, failureListener, DEFAULT_EXECUTOR);
	}

	public static <T> void call(Task<T> task, ChangeListener<T> successHandler, ChangeListener<Throwable> failureHandler) {
		call(task, successHandler, failureHandler, DEFAULT_EXECUTOR);
	}


	public static <T> void call(Task<T> task, Executor executor) {
		call(task, Handlers.<T>noOpSuccessHandler(), noOpFailureHandler());
	}

	public static <T> void call(Task<T> task, ChangeListener<T> successHandler, Executor executor) {
		call(task, successHandler, noOpFailureHandler());
	}

	public static <T> void call(Task<T> task, InvalidationListener successListener, Executor executor) {
		call(task, successListener, noOpListener());
	}



	public static <T> void call(Task<T> task, InvalidationListener successListener, InvalidationListener failureListener, Executor executor) {
		task.valueProperty().addListener(successListener);
		task.exceptionProperty().addListener(failureListener);
		executor.execute(task);
	}

	public static <T> void call(Task<T> task, ChangeListener<T> successHandler, ChangeListener<Throwable> failureHandler, Executor executor) {
		task.valueProperty().addListener(successHandler);
		task.exceptionProperty().addListener(failureHandler);
		executor.execute(task);
	}
}
