package com.broodsoft.grandeur.activity;

import javafx.stage.Stage;

import com.broodsoft.id.Id;
import com.google.common.eventbus.EventBus;
import com.google.inject.Inject;
import com.google.inject.Provider;

public abstract class AbstractActivity implements Activity {
	private final Id id;
	private final String title;
	private final Stage stage;

	protected final EventBus nexusEventBus;
	protected final EventBus activityEventBus;

	protected AbstractActivity(Configuration config, String title) {
		this.id = config.id;
		this.nexusEventBus = config.nexusEventBus;
		this.title = title;

		if(config.stage == null || config.stage.getScene() == null)
			throw new IllegalArgumentException("Neither stage nor scene are allowed to be null");
		this.stage = config.stage;

		activityEventBus = new EventBus();
		this.stage.setTitle(title);
	}


	public Id getId(){ return id; }
	public Stage getStage(){ return stage; }
	public EventBus getActivityEventBus(){ return activityEventBus; }

	public String getTitle(){ return title; }

	public void show(){ stage.show(); }
	public void hide(){ stage.hide(); }



	public static final class Configuration {
		private final Id id;
		private final Stage stage;
		private final EventBus nexusEventBus;

		public Configuration(Id id, Stage stage, EventBus nexusEventBus) {
			this.id = id;
			this.stage = stage;
			this.nexusEventBus = nexusEventBus;
		}
	}

	public static final class ConfigurationProvider implements Provider<Configuration> {
		private final Configuration config;

		@Inject
		public ConfigurationProvider(Id id, Stage stage, EventBus nexusEventBus) {
			config = new Configuration(id, stage, nexusEventBus);
		}

		public Configuration get(){ return config; }
	}
}
