package com.broodsoft.grandeur.activity.naming;

import java.net.URL;

import com.google.common.base.Optional;
import com.google.common.base.Strings;

public abstract class ActivityResourceNamingStrategy<T> implements ResourceNamingStrategy<T> {
	protected static final String DEFAULT_SUFFIX = "Activity";

	protected final String expectedSuffix;
	protected final String replacementSuffix;
	protected final Optional<String> fileExtension;

	protected ActivityResourceNamingStrategy(String replacementSuffix) {
		this(DEFAULT_SUFFIX, replacementSuffix, Optional.<String>absent());
	}

	protected ActivityResourceNamingStrategy(String replacementSuffix, String fileExtension) {
		this(DEFAULT_SUFFIX, replacementSuffix, Optional.fromNullable(Strings.emptyToNull(fileExtension)));
	}

	protected ActivityResourceNamingStrategy(String expectedSuffix, String replacementSuffix, Optional<String> fileExtension) {
		this.expectedSuffix = expectedSuffix;
		this.replacementSuffix = replacementSuffix;
		this.fileExtension = fileExtension;
	}

	public String getName(Class<?> type) throws NoSuchResourceException {
		final String name = stripSuffix(type, DEFAULT_SUFFIX);
		if(fileExtension.isPresent()) {
			return name + replacementSuffix + "." + fileExtension.get();
		} else {
			return name + replacementSuffix;
		}
	}

	protected URL getLocation(Class<?> type) throws NoSuchResourceException {
		final String name = getName(type);
		final URL resource = type.getResource(name);
		if(resource == null) {
			throw new NoSuchResourceException("Failed to locate a resource named '"+name+"' for "+type.getCanonicalName());
		}
		return resource;
	}

	private String stripSuffix(Class<?> type, String suffix) throws NoSuchResourceException {
		final String name = type.getSimpleName();
		final int indexOfSuffix = name.lastIndexOf(suffix);

		if(indexOfSuffix <= 0) {
			throw new NoSuchResourceException("Invalid type name. The name '"+
				name+"' doesn't contain the suffix "+"'"+suffix+"'");
		} else {
			return name.substring(0, indexOfSuffix);
		}
	}
}
