package com.broodsoft.grandeur.intent;

import com.broodsoft.grandeur.activity.Activity;

public class CreateActivityIntent<A extends Activity> implements ActivityLifecycleIntent {
	public static interface Handler<A extends Activity> {
		void handleCreated(A activity);
	}

	public static final Handler<? extends Activity> NO_OP_HANDLER = new Handler<Activity>() {
		@Override public void handleCreated(Activity activity) {}
	};

	public static final boolean DEFAULT_DO_SHOW_VALUE = true;

	private final Class<A> activityType;
	private final Handler<A> creationHandler;
	private final boolean doShow;

	public CreateActivityIntent(Class<A> activityType) {
		this(activityType, CreateActivityIntent.<A>noOpHandler());
	}

	public CreateActivityIntent(
		Class<A> activityType,
		Handler<A> creationHandler
	) {
		this(activityType, creationHandler, DEFAULT_DO_SHOW_VALUE);
	}

	public CreateActivityIntent(
		Class<A> activityType,
		boolean doShow
	) {
		this(activityType, CreateActivityIntent.<A>noOpHandler(), doShow);
	}

	public CreateActivityIntent(
		Class<A> activityType,
		Handler<A> launchHandler,
		boolean doShow
	) {
		this.activityType = activityType;
		this.creationHandler = launchHandler;
		this.doShow = doShow;
	}
	
	public Class<A> getActivityType() {
		return activityType;
	}

	public Handler<A> getCreationHandler() {
		return creationHandler;
	}

	public boolean doShow() {
		return doShow;
	}

	@SuppressWarnings("unchecked")
	public static <A extends Activity> Handler<A> noOpHandler() {
		return (Handler<A>)NO_OP_HANDLER;
	}
}
