package com.broodsoft.guice;

import javafx.scene.Group;
import javafx.scene.Scene;
import javafx.stage.Stage;

import com.broodsoft.grandeur.activity.AbstractActivity;
import com.broodsoft.grandeur.activity.SimpleFxmlActivity;
import com.broodsoft.grandeur.activity.naming.CssFileNamingStrategy;
import com.broodsoft.grandeur.activity.naming.DefaultCssFileNamingStrategy;
import com.broodsoft.grandeur.activity.naming.DefaultFxmlFileNamingStrategy;
import com.broodsoft.grandeur.activity.naming.PropertiesFileNamingStrategy;
import com.broodsoft.grandeur.activity.naming.FxmlFileNamingStrategy;
import com.broodsoft.grandeur.activity.naming.ResourceBundleFileNamingStrategy;
import com.google.common.collect.ImmutableList;
import com.google.inject.AbstractModule;
import com.google.inject.Provides;

public class ApplicationInjectionModule extends AbstractModule {
	private final ImmutableList<String> stylesheets;

	public ApplicationInjectionModule() {
		this(ImmutableList.<String>of());
	}

	public ApplicationInjectionModule(ImmutableList<String> stylesheets) {
		this.stylesheets = stylesheets;
	}

	@Override
	protected void configure() {
		bind(AbstractActivity.Configuration.class).toProvider(AbstractActivity.ConfigurationProvider.class);
		bind(SimpleFxmlActivity.Configuration.class);

		configureNamingStrategies();
	}

	protected void configureNamingStrategies() {
		bind(FxmlFileNamingStrategy.class).to(DefaultFxmlFileNamingStrategy.class);
		bind(ResourceBundleFileNamingStrategy.class).to(PropertiesFileNamingStrategy.class);
		bind(CssFileNamingStrategy.class).to(DefaultCssFileNamingStrategy.class);
	}

	@Provides
	protected Stage createStage() {
		Stage stage = new Stage();
		Scene scene = new Scene(new Group());
		scene.getStylesheets().addAll(getGlobalStyleSheets());
		stage.setScene(scene);
		return stage;
	}

	protected ImmutableList<String> getGlobalStyleSheets() {
		return stylesheets;
	}
}
