package com.broodsoft.grandeur.demo.activity.login;

import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.scene.Node;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.PasswordField;
import javafx.scene.control.TextField;
import javafx.scene.input.KeyCode;
import javafx.scene.input.KeyEvent;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.broodsoft.grandeur.activity.SimpleFxmlActivity;
import com.broodsoft.grandeur.demo.session.Session;
import com.broodsoft.grandeur.intent.EventFromSource;
import com.google.inject.Inject;

public class LoginActivity extends SimpleFxmlActivity {
	private static final Logger LOGGER = LoggerFactory.getLogger(LoginActivity.class);

	private static final String VALID_USERNAME = "admin";
	private static final String VALID_PASSWORD = "foobar";

	public class SuccessNotification extends EventFromSource<LoginActivity> {
		public final Session session;

		private SuccessNotification(Session session) {
			super(LoginActivity.this);
			this.session = session;
		}
	}

	@FXML private Node root;
	@FXML private Label hintDisplay;
	@FXML private TextField usernameInput;
	@FXML private PasswordField passwordInput;
	@FXML private Button loginControl;

	@Inject
	protected LoginActivity(SimpleFxmlActivity.Configuration config) {
		super(config, "Login");
	}

	@FXML
	private void handleLoginClick(ActionEvent event) {
		attemptLogin();
	}

	private void attemptLogin() {
		final String username = usernameInput.getText();
		final String password = passwordInput.getText();
		LOGGER.debug("Attemping login using: "+username+"/"+password);

		final boolean isLoggedIn = areCredentialsValid(username, password);

		if(isLoggedIn) {
			final Session session = new Session(username);
			activityEventBus.post(new SuccessNotification(session));
		}
	}

	@FXML
	private void initialize() {
		LOGGER.debug("Initializing");
		hintDisplay.setText("Valid credentials: "+VALID_USERNAME+"/"+VALID_PASSWORD);
		root.addEventHandler(KeyEvent.KEY_RELEASED, new EventHandler<KeyEvent>(){
			public void handle(KeyEvent event) {
				if(event.getCode().equals(KeyCode.ENTER))
					attemptLogin();
			}
		});
	}

	private boolean areCredentialsValid(String username, String password) {
		return username.equals(VALID_USERNAME) && password.equals(VALID_PASSWORD);
	}
}
