package com.broodsoft.grandeur.demo.navigation;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.broodsoft.grandeur.demo.activity.login.LoginActivity;
import com.broodsoft.grandeur.demo.activity.main.MainActivity;
import com.broodsoft.grandeur.demo.session.Session;
import com.broodsoft.grandeur.intent.CreateActivityIntent;
import com.broodsoft.grandeur.intent.DestroyActivityIntent;
import com.broodsoft.grandeur.navigation.Navigator;
import com.broodsoft.grandeur.nexus.Nexus;
import com.broodsoft.guice.MutableProvider;
import com.google.common.eventbus.Subscribe;
import com.google.inject.Inject;

public class LoginToMainNavigator implements Navigator {
	private static final Logger LOGGER = LoggerFactory.getLogger(LoginToMainNavigator.class);

	private final Nexus nexus;
	private final MutableProvider<Session> sessionProvider;

	private LoginActivity loginActivity;

	@Inject
	public LoginToMainNavigator(Nexus nexus, MutableProvider<Session> sessionProvider) {
		this.nexus = nexus;
		this.sessionProvider = sessionProvider;
	}

	public void navigate() {
		LOGGER.debug("Navigator for: "+LoginActivity.class.getSimpleName()+" --> "+MainActivity.class.getSimpleName());
		nexus.getNexusEventBus().post(new CreateActivityIntent<>(
			LoginActivity.class,
			new CreateActivityIntent.Handler<LoginActivity>() {
				public void handleCreated(LoginActivity login) {
					loginActivity = login;
					loginActivity.getActivityEventBus().register(LoginToMainNavigator.this);
				}
			}
		));
	}

	@Subscribe
	public void handleLoginSuccessNotification(LoginActivity.SuccessNotification notification) {
		LOGGER.debug("Navigating to "+MainActivity.class.getSimpleName());
		final Session session = notification.session;
		sessionProvider.set(session);
		nexus.getNexusEventBus().post(new CreateActivityIntent<>(
			MainActivity.class,
			new CreateActivityIntent.Handler<MainActivity>() {
				public void handleCreated(MainActivity main) {
					loginActivity.getActivityEventBus().unregister(LoginToMainNavigator.this);
					nexus.getNexusEventBus().post(new DestroyActivityIntent(loginActivity.getId()));
				}
			}
		));
	}
}
